/*
 *  Interfejs do historii wiadomoci ekg - funkcje klasy rozmowy.
 *  Copyright (C) 2004 Piotr Kalinowski <pitkali@o2.pl> All rights reserved.
 *  Biblioteka podlega licencji GNU GPL.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string>
#include "ekglog.hpp"
#include "regexp.hpp"

using namespace EkgLog;
using namespace std;

// Szukaj wyraenia regularnego w rozmowie. Szczegowa dokumentacja jest w ekglog.hpp
// TODO: pono obecnie przy fladze ignore_case ustawionej na true, trzeba wyraenia 
// podawa wycznie za pomoc maych liter... Trzeba to bdzie poprawi.
SearchResult Chat::search (const string& regex, const bool ignore_case)
throw(runtime_error)
{
  msg_iterator iter;
  RegExp regexp (true);
  reg_syntax_t syntax = re_syntax_options;
  if (ignore_case)
    syntax &= RE_ICASE;
  regexp.compile (regex, syntax);
  if (!regexp.is_compiled())
    throw runtime_error ("Bd przy kompilacji wyraenia regularnego: " + regexp.get_error());
  int result;
  for (iter = _begin; iter != _end; ++iter) {
    result = regexp.search (iter->msg());
    if (result >= 0) // a match !
      return SearchResult (iter, result, regexp.get_match().end()-result);
  }
  return SearchResult (iter, string::npos, string::npos);
}

SearchResult Chat::search (const string& regex, const SearchResult& prev,
    const bool ignore_case) throw(std::runtime_error)
{
  msg_iterator iter = prev.message();
  RegExp regexp (true);
  reg_syntax_t syntax = re_syntax_options;
  if (ignore_case)
    syntax &= RE_ICASE;
  regexp.compile (regex,syntax);
  if (!regexp.is_compiled())
    throw runtime_error ("Bd przy kompilacji wyraenia regularnego: " + regexp.get_error());
  int result = regexp.search (iter->msg(), prev.begin() + prev.length());
  if (result >= 0) // a match!
    return SearchResult (iter, result, regexp.get_match().end() - result);
  for (++iter; iter != _end; ++iter) {
    result = regexp.search (iter->msg());
    if (result >= 0)
      return SearchResult (iter,result, regexp.get_match().end() - result);
  }
  return SearchResult (iter, string::npos, string::npos);
}

