/*  Interfejs do odczytu historii wiadomoci programu ekg - funkcje klasy uin.
 *  Copyright (C) 2004 Piotr Kalinowski <pitkali@o2.pl> All rights reserved.
 *  Niniejsza biblioteka podlega licencji GNU GPL.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "ekglog.hpp"
#include <utility>

using namespace EkgLog;

// ustaw list czatw na podstawie odlegoci (w czasie) wiadomoci.
void UinLog::fillChats (const time_t interval)
{
  // a jeli nie ma adnych wiadomoci?
  if (_msg.begin() == _msg.end())
    return; // po prostu wyjd
  if (!_chats.empty())
  // usu obecny podzia na czaty
    _chats.clear();
  std::list<Message>::iterator beg, end, i;
  Chat chat; // czat tymczasowy - jego kopie utworz list czatw
  unsigned int lp = 2; // do indeksowania wiadomoci w czacie

  // beg - pierwsza wiadomo w biecym czacie
  // end - potencjalnie ostatnia wiadomo w czacie
  // i - jest o jedn pozycj dalej ni end - jeli zakwalifikuje si
  //     jako pocztek nowego czatu, bdzie to nowy beg
  beg = end = i = _msg.begin();
  beg->set_no (1);
  ++i;
  while (i != _msg.end()) {
    // niezmiennik: ++end == i
    if (i->recv() - end->recv() >= interval) {
      ++end; // czat przechowuje indeks za-kocowy!
      chat.begin (beg);
      chat.end (end);
      _chats.insert (std::make_pair (chat.time(),chat));
      beg = end;
      beg->set_no (1);
      lp = 2;
      ++i;
    } else {
      i->set_no (lp++);
      ++end;
      ++i;
    }
  }
  // iterator i doszed do koca listy wiadomoci. Warunek if tworzcy
  // nowy czat nie zosta speniony dla ostatniego czata (bo skoczyly si
  // wiadomoci i zabrako tej, ktra by spowodowaa wstawienie czata do listy.
  ++end;
  chat.begin (beg);
  chat.end (end);
  _chats.insert (std::make_pair (chat.time(),chat));
} // fillChats
  
